/*
 * MyGraph.java
 *
 * Created on April 11, 2004, 10:41 PM
 * this class store a graph in an adjacent Matrix
 * a vector is used to store the vertices and it's coordinate
 * a matrix is used to store edge
 * upon creating of the instance, the node list and adjacent must be passed in
 
 */

package MySimulation;
import java.util.regex.*;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.*;
import java.util.TreeMap;
import java.awt.geom.*;

/**
 *
 * @author  Kevin Chiou
 */
public class MyGraph {
    Vector nodes = new Vector(); // nodes to track roadmaps
    boolean edges[][];
    int w[][] = null;  // a integer array to hold the weights of each edges
    int pred[] = null; // predecesor array
    int d[] = null; // distance matrix for shortest path algorithm
    int color[] = null; // track if the particular nodes been visited
    
    
    
    /** Creates a new instance of MyGraph */
    public MyGraph(Vector nodesin, boolean[][] edgesin) {
        nodes = nodesin;
        edges = edgesin;
        w = allocateIntegerArray(nodes.size(),nodes.size(), 0); // allocate a weight graph for the edges
                                                           // default weights to 1    
        // set weight to distance of the two points
        for(int i =0; i < nodes.size(); i++)
            for(int j = 0 ; j < nodes.size(); j++)
            {
                Point2D pt1 = (Point2D) nodes.get(i);
                Point2D pt2 = (Point2D) nodes.get(j);
                w[i][j] = (int) pt1.distance(pt2);
            }
        
        
        d = new int[nodes.size()];
        pred = new int[nodes.size()];
        color = new int[nodes.size()];
        
    }
   // disjkstra's algorithm based on Dr. Mount's class note for CMSC451
    
    public void Dijkstra(int s)
    // s: source
    {
        PriorityQueue Q = new PriorityQueue();
        int u = 0;
        
        init_Dijkstra();    // initialize all the datas
        d[s] = 0;
        // put all vertices in the queue
      /*  for(int i = 0; i< nodes.size(); i++)
        {
            Q.insert(new Integer(d[i]), nodes.get(i));
        }
        */
        while(Q.nonEmpty())
        {
            u = Q.extractMin();
            for(int v = 0; v< nodes.size(); v++)
            {
                if (edges[u][v]) // if node v is u's neighbor
                {
                   Relax(u,v);                    
                }
            }
            color[u] = 1;
        }
    }
    
    public int getDistance(int i)
    {return d[i];}
    
    private void Relax(int u, int v)
    {
        //System.out.println(u + "," + v);
        if (d[u] + w[u][v] < d[v])
        {
            d[v] = Math.abs(d[u] + w[u][v]);
            pred[v] = u;
        }
    }
    
    private void init_Dijkstra()
    // initialize data for Dijkstra's algorithm
    {
        for(int i = 0; i< nodes.size(); i++)
        {
            d[i] = Integer.MAX_VALUE;
            color[i] = 0; // 0 = white = unvisited
            pred[i] = -1; // invalid node            
        }               
    }
    
   
    
    //********************** helper functions
    
    private int[][] allocateIntegerArray(int maxx, int maxy, int weight) {
        /// allocating space for the new array, and fill with false (no link)
        // weight is the default weight for the edges
         int newarray[][];
         newarray = new int[maxx][];
        for(int i = 0; i< maxx; i++) {
            newarray[i] = new int[maxy];
        }
        for(int i = 0; i < maxx; i++)
            for (int j =0; j<maxy; j++)
                newarray[i][j] = weight;
        return newarray;
    }
    
    private class PriorityQueue
    {
        public int extractMin()
        {
            int min = -1;
            int mind = Integer.MAX_VALUE;
            
            for(int i  = 0; i< nodes.size(); i++)
            {
                if ( (color[i] == 0) && d[i] <= mind)  // unvisited and with shorter distance
                {
                    min = i;
                    mind = d[i];
                }
            }
            
            return min;
        }
        
        public boolean nonEmpty()
        {
              for(int i  = 0; i< nodes.size(); i++)
            {
                if (color[i] == 0) // if any node is unvisited, then nonEmpty is false
                {
                    return true;
                }
            }
            
            return false;
        }
        
        public void decreaseKey(int node, int newdistance)
        {
            d[node] = newdistance;
        }
        
        /*TreeMap queue= new TreeMap();
        
        public void insert(Integer distance, Object node)
        {                     
            queue.put( distance,node);
        }
        
        public int extractMin()
        // find minimal value and delete from the queue and return it
        {
            return ((Integer) queue.remove(queue.firstKey())).intValue(); // find the minimal                       
        }
        
        boolean nonEmpty()
        {
            return !queue.isEmpty();
        }
        
        public void decreaseKey(Object node, int newdistance)
        {
        }
         **/
    }
    
}
